﻿// --------------------------------------------------------------------------------------
//
// Ex5Main.cs                                                     Ulrich Müller  15.05.08
//                                                                PCS1700 C# 2005 Express
// Explorer : Bedienoberfläche für den ROBO Explorer über das RF Datalink
//            Zusätzlich ein AUTOPILOT für das Fahren a la Explorer3
//            Basis ist die Assembly FishFace2005.DLL v4.3.75.2005
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using FishFace40;

namespace Explorer5 {
  public partial class Explorer5 : Form {
    FishFace ft = new FishFace();
    Thread AutoPilot;
    Thread Licht;
    Thread Temperatur;
    Thread Farberkennung;

    bool AufSpur;
    bool LichtAn = false;

    // --- Interfacebelegung ---

    const Out mLinks       = Out.M2;
    const Out mRechts      = Out.M1;

    const Out oLampeR      = Out.O5;
    const Out oLampeL      = Out.O6;
    const Out oLampeT      = Out.O7;
    const Out oHupe        = Out.O8;

    const Dir cVor         = Dir.Left;
    const Dir cRuck        = Dir.Right;
    const Dir cOff         = Dir.Off;

    const Inp iImpulsLinks = Inp.I1;
    const Inp iSpurRechts  = Inp.I3;
    const Inp iSpurLinks   = Inp.I4;

    const Inp dUltra       = Inp.D1;

    const Inp aLicht       = Inp.AX;
    const Inp aTemp        = Inp.AY;

    const Inp aFarbe       = Inp.A1;

    // --- Farbkonstanten ----

    const int blauMin = 240;
    const int blauMax = 260;
    const int grunMin = 200;
    const int grunMax = 220;
    const int rotMin  = 170;
    const int rotMax  = 190;

    const int heissMax = 350;

    // --- Unterprogramme Fahren ---

    private void Stopp() {
      ft.SetMotor(mLinks,  cOff);
      ft.SetMotor(mRechts, cOff);
    }
    private void StoppR() {
      ft.SetMotor(mLinks,  cVor);
      ft.SetMotor(mRechts, cOff);
    }
    private void StoppL() {
      ft.SetMotor(mLinks,  cOff);
      ft.SetMotor(mRechts, cVor);
    }
    private void Vorwarts() {
      ft.SetMotor(mLinks,  cVor, Speed.Half);
      ft.SetMotor(mRechts, cVor, Speed.Half);
    }
    private void Ruckwarts() {
      ft.SetMotor(mLinks,  cRuck, Speed.Half);
      ft.SetMotor(mRechts, cRuck, Speed.Half);
    }
    private void Links() {
      ft.SetMotor(mLinks,  cRuck, Speed.Half);
      ft.SetMotor(mRechts, cVor, Speed.Half);
    }

    // --- Hindernis : Wenn Entfernung zu einem Hindernis größer 12 -> return
    //     sonst rückwärts bis Entfernung > 13 cm, dann 500 ms LinksDrehen
    private bool Hindernis() {
        if (ft.GetDistance(dUltra) >= 12) return false;
        Stopp();
        ft.Pause(1000);
        Ruckwarts();
        while (!ft.Finish()
          && (ft.GetDistance(dUltra) <= 13)) ft.Pause(123);
        Stopp();
        Links();
        ft.Pause(500);
        return true;
    }

    // --- Spursuche : Wenn ein SpurSensor auf Spur : return
    //     sonst EndlosSuche 52x High Links und 10x High Vorwärts
    private void Spursuche() {
      int Zahler;
      AufSpur = false;
      while (!ft.Finish()) {
        Zahler = 52;
        Links();
        do {
          if (!ft.GetInput(iSpurLinks) || !ft.GetInput(iSpurRechts)) {
            Stopp(); AufSpur = true; return;
          }
          ft.WaitForHigh(iImpulsLinks);
          if (Hindernis()) { Zahler = 52; Links(); } else Zahler--;
        } while (Zahler > 0 && !ft.Finish());
        Zahler = 10;
        Vorwarts();
        do {
          if (!ft.GetInput(iSpurLinks) || !ft.GetInput(iSpurRechts)) {
            Stopp(); AufSpur = true; return;
          }
          ft.WaitForHigh(iImpulsLinks);
          if (Hindernis()) Zahler = 0; else Zahler--;
        } while (Zahler > 0 && !ft.Finish());
      } 
    }

    // --- Thread Main-Routinen --------------------

    // --- AutoLoop Fahren unter Auswertung der SpurSensorwerte ---
    private void AutoLoop() {
      while (!ft.Finish()) {
        switch (ft.GetInputs() & 0xC) {
          case 0xC:
            Stopp(); Spursuche(); break;
          case 0x8:
            StoppL(); Hindernis(); break;
          case 0x4:
            StoppR(); Hindernis(); break;
          default: Vorwarts(); Hindernis(); break;
        }
      }
    }

    // --- LichtLoop : Schalten des Fahrlichtes - ein bei Helligkeitswert > 500 ---
    private void LichtLoop() {
      while(!ft.Finish()) {
        if (ft.GetAnalog(aLicht) > 500) {
          ft.SetLamp(oLampeL, Dir.On); ft.SetLamp(oLampeR, Dir.On);
        }
        else {
          ft.SetLamp(oLampeL, Dir.Off); ft.SetLamp(oLampeR, Dir.Off);
        }
      }
    } 

    // --- TempLoop : Temperaturkontrolle - RoteLampe, wenn Wert < 300 ---
    private void TempLoop() {
      while (!ft.Finish()) {
        if (ft.GetAnalog(aTemp) < heissMax) {
          ft.SetLamp(oLampeT, Dir.On);
          ft.Pause(100);
          ft.SetLamp(oLampeT, Dir.Off);
          ft.Pause(600);
        }
      }
    }

    private void Hupen(int nMal, int Dauer) {
      for (int i = 0; i < nMal; i++) {
        ft.SetLamp(oHupe, Dir.On);
        ft.Pause(Dauer);
        ft.SetLamp(oHupe, Dir.Off);
        ft.Pause(Dauer);
      }
    }
    // --- FarbLoop : Farberkennung, wenn Explorer AufSpur ---
    private void FarbLoop() {
      int farbe;
      while(!ft.Finish()) {
        if (AufSpur) {
          farbe = ft.GetVoltage(aFarbe);
          if (farbe > blauMin & farbe < blauMax) Hupen(4, 100);
          if (farbe > grunMin & farbe < grunMax) Hupen(3, 300);
          if (farbe > rotMin & farbe < rotMax) Hupen(2, 600);
        }
      }
    }

    private void Action() {
      // --- Routine für den Modellbetrieb, kann weitere Methoden aufrufen ---
      do {
        lblAbstand.Text    = ft.GetDistance(Inp.D1).ToString();
        lblFarbwert.Text   = ft.GetVoltage(Inp.A1).ToString();
        lblHelligkeit.Text = ft.GetAnalog(Inp.AX).ToString();
        lblTemperatur.Text = ft.GetAnalog(Inp.AY).ToString();
        lblSpurL.Text      = ft.GetInput(Inp.I3) ? "F" : "T";
        lblSpurR.Text      = ft.GetInput(Inp.I4) ? "F" : "T";
        ft.Pause(77);
      } while (!ft.Finish());
    }

    #region --- Programm-Kontrolle ---
    public Explorer5() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      try {
        ft.OpenInterface(IFTypen.ftROBO_first_USB, 0, Distance.UltraSonic, true);
        cmdAction.Enabled    = false;
        chkAutoPilot.Enabled = true;
        chkAutoPilot.Checked = false;
        pnlDrive.Enabled     = true;
        cmdEnde.Text         = "&HALT";
        lblHinweis.Text      = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();

        ft.ClearMotors();
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        Thread.Sleep(555);
        ft.CloseInterface();
        cmdAction.Enabled    = true;
        chkAutoPilot.Enabled = false;
        pnlDrive.Enabled     = false;
        cmdEnde.Text         = "&ENDE";
        lblHinweis.Text      = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      if (cmdEnde.Text == "&HALT") {
        ft.NotHalt           = true; 
        chkAutoPilot.Checked = false;
      }
      else this.Close();
    }

    private void formLoad(object sender, EventArgs e) {
      // --- Default-Interface : Erstes ROBO Interface an USB ----
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion

    // --- Routinen für die Fahr-Buttons -------------------------

    private void cmdVor_MouseDown(object sender, MouseEventArgs e) {
      ft.SetMotor((int)Out.M2, (int)Dir.Left, (int)numGeschwindigkeit.Value);
      ft.SetMotor((int)Out.M1, (int)Dir.Left, (int)numGeschwindigkeit.Value);
    }
    private void cmdVor_MouseUp(object sender, MouseEventArgs e) {
      Stopp();
    }

    private void cmdZuruck_MouseDown(object sender, MouseEventArgs e) {
      ft.SetMotor((int)Out.M2, (int)Dir.Right, (int)numGeschwindigkeit.Value);
      ft.SetMotor((int)Out.M1, (int)Dir.Right, (int)numGeschwindigkeit.Value);
    }
    private void cmdZuruck_MouseUp(object sender, MouseEventArgs e) {
      Stopp();
    }

    private void cmdLinks_MouseDown(object sender, MouseEventArgs e) {
      ft.SetMotor((int)Out.M2, (int)Dir.Right, (int)numGeschwindigkeit.Value);
      ft.SetMotor((int)Out.M1, (int)Dir.Left, (int)numGeschwindigkeit.Value);
    }
    private void cmdLinks_MouseUp(object sender, MouseEventArgs e) {
      Stopp();
    }

    private void cmdRechts_MouseDown(object sender, MouseEventArgs e) {
      ft.SetMotor((int)Out.M2, (int)Dir.Left, (int)numGeschwindigkeit.Value);
      ft.SetMotor((int)Out.M1, (int)Dir.Right, (int)numGeschwindigkeit.Value);
    }
    private void cmdRechts_MouseUp(object sender, MouseEventArgs e) {
      Stopp();
    }

    // --- Routinen Licht und Hupe Buttons --------------------

    private void cmdLicht_Click(object sender, EventArgs e) {
      if (LichtAn) {
        ft.SetLamp(Out.O5, Dir.Off);
        ft.SetLamp(Out.O6, Dir.Off);
        cmdLicht.BackColor = Color.LightBlue;
        LichtAn = false;
      }
      else {
        ft.SetLamp(Out.O5, Dir.On);
        ft.SetLamp(Out.O6, Dir.On);
        cmdLicht.BackColor = Color.LightPink;
        LichtAn = true;
      }
    }

    private void cmdHupe_MouseDown(object sender, MouseEventArgs e) {
      ft.SetLamp(Out.O8, Dir.On);
    }
    private void cmdHupe_MouseUp(object sender, MouseEventArgs e) {
      ft.SetLamp(Out.O8, Dir.Off);
    }

    // --- Starten / Beenden des AutoPilot Modes ----------------------

    private void chkAutoPilot_CheckedChanged(object sender, EventArgs e) {
      if (chkAutoPilot.Checked) {
        pnlDrive.Enabled = false;
        ft.NotHalt = false;
        chkAutoPilot.BackColor = Color.Red;
        AutoPilot = new Thread(AutoLoop);
        AutoPilot.IsBackground = true;
        Licht = new Thread(LichtLoop);
        Licht.IsBackground = true;
        Temperatur = new Thread(TempLoop);
        Temperatur.IsBackground = true;
        Farberkennung = new Thread(FarbLoop);
        Farberkennung.IsBackground = true;
        AutoPilot.Start();
        Licht.Start();
        Temperatur.Start();
        Farberkennung.Start();
      }
      else {
        ft.NotHalt = true;
        pnlDrive.Enabled = true;
        chkAutoPilot.BackColor = Color.Gray;
        Thread.Sleep(123);
        ft.ClearMotors();
      }
    }
  }
}