﻿// --------------------------------------------------------------------------------------
//
// Ex5dMain.cs                                                    Ulrich Müller  27.05.08
//                                                                PCS1700 C# 2005 Express
// Explorer : Bedienoberfläche für den ROBO Explorer über das RF Datalink
//            Zusätzlich ein AUTOPILOT für das Fahren a la Explorer3
//            Genutzt wird die Klasse Explorer5
//            Basis ist die Assembly FishFace2005.DLL v4.3.75.2005
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Explorer5;
using FishFace40;

namespace Explorer5d {
  public partial class Explorer5d : Form {
    Explorer ex = new Explorer();

    bool AufSpur;
    bool LichtAn = false;

    private void MessDaten(object sender, DatenEingaenge Werte) {
        lblAbstand.Text    = Werte.Distance.ToString();
        lblFarbwert.Text   = Werte.Farbe.ToString();
        lblHelligkeit.Text = Werte.Helligkeit.ToString();
        lblTemperatur.Text = Werte.Temperatur.ToString();
        lblSpurL.Text      = Werte.SpurL ? "F" : "T";
        lblSpurR.Text      = Werte.SpurR ? "F" : "T";
    }

    #region --- Programm-Kontrolle ---
    public Explorer5d() {
      InitializeComponent();
      Label.CheckForIllegalCrossThreadCalls = false;
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      try {
        ex.WerteEingaenge += new Explorer.Messwerte(MessDaten);
        ex.OpenExplorer(IFTypen.ftROBO_first_USB, 0);
        cmdAction.Enabled    = false;
        chkAutoPilot.Enabled = true;
        chkAutoPilot.Checked = false;
        pnlDrive.Enabled     = true;
        cmdEnde.Text         = "&HALT";
        lblHinweis.Text      = "--- Ende : HALT-Button oder ESC-Taste ---";
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      if (cmdEnde.Text == "&HALT") {
        ex.CloseExplorer();
        cmdAction.Enabled = true;
        chkAutoPilot.Enabled = false;
        pnlDrive.Enabled = false;
        cmdEnde.Text = "&ENDE";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
      else this.Close();
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion

    // --- Routinen für die Fahr-Buttons -------------------------

    private void cmdVor_MouseDown(object sender, MouseEventArgs e) {
      ex.Vorwaerts((Speed)numGeschwindigkeit.Value);
    }
    private void cmdVor_MouseUp(object sender, MouseEventArgs e) {
      ex.Stopp();
    }

    private void cmdZuruck_MouseDown(object sender, MouseEventArgs e) {
      ex.Rueckwaerts((Speed)numGeschwindigkeit.Value);
    }
    private void cmdZuruck_MouseUp(object sender, MouseEventArgs e) {
      ex.Stopp();
    }

    private void cmdLinks_MouseDown(object sender, MouseEventArgs e) {
      ex.DrehenLinks((Speed)numGeschwindigkeit.Value);
    }
    private void cmdLinks_MouseUp(object sender, MouseEventArgs e) {
      ex.Stopp();
    }

    private void cmdRechts_MouseDown(object sender, MouseEventArgs e) {
      ex.DrehenRechts((Speed)numGeschwindigkeit.Value);
    }
    private void cmdRechts_MouseUp(object sender, MouseEventArgs e) {
      ex.Stopp();
    }

    // --- Routinen Licht und Hupe Buttons --------------------

    private void cmdLicht_Click(object sender, EventArgs e) {
      if (LichtAn) {
        ex.LichtAus();
        cmdLicht.BackColor = Color.LightBlue;
        LichtAn = false;
      }
      else {
        ex.LichtAn();
        cmdLicht.BackColor = Color.LightPink;
        LichtAn = true;
      }
    }

    private void cmdHupe_MouseDown(object sender, MouseEventArgs e) {
      ex.HupeEin();
    }
    private void cmdHupe_MouseUp(object sender, MouseEventArgs e) {
      ex.HupeAus();
    }

    // --- Starten / Beenden des AutoPilot Modes ----------------------

    private void chkAutoPilot_CheckedChanged(object sender, EventArgs e) {
      if (chkAutoPilot.Checked) {
        pnlDrive.Enabled = false;
        ex.StartFarbUeberwachung();
        ex.StartLichtUeberwachung();
        ex.StartTemperaturUeberwachung();
        ex.StartSpurfahren();
        chkAutoPilot.BackColor = Color.Red;
      }
      else {
        ex.StopSpurfahren();
        ex.StopFarbUeberwachung();
        ex.StopLichtUeberwachung();
        ex.StopTemperaturUeberwachung();
        pnlDrive.Enabled = true;
        chkAutoPilot.BackColor = Color.Gray;
      }
    }
  }
}