﻿' -----------------------------------------------------------------------------------------
'
' Explorer5vb                                                       Ulrich Müller  19.05.08
'                                                                   PCS1700 VB 2005 Express
' Bedienoberfläche für den ROBO Explorer über das RF Datalink
' Zusätzlich ein AUTOPILOT für das Fahren a la Explorer3
'   Basis ist die Assembly FishFace2005.DLL
'
' -----------------------------------------------------------------------------------------

Option Explicit On
Option Strict Off
Imports System.Threading
Imports FishFace40

Public Class Explorer5vb

  Dim ft As New FishFace()
  Dim AutoPilot As Thread
  Dim Licht As Thread
  Dim Temperatur As Thread
  Dim Farberkennung As Thread

  Dim AufSpur As Boolean
  Dim LichtAn As Boolean = False

  ' --- Interfacebelegung ----------

  Const mLinks As Out = Out.M2
  Const mRechts As Out = Out.M1

  Const oLampeR As Out = Out.O5
  Const oLampeL As Out = Out.O6
  Const oLampeT As Out = Out.O7
  Const oHupe As Out = Out.O8

  Const cVor As Dir = Dir.Left
  Const cRuck As Dir = Dir.Right
  Const cOn As Dir = Dir.On
  Const cOff As Dir = Dir.Off

  Const iImpulsLinks As Inp = Inp.I1
  Const iSpurRechts As Inp = Inp.I3
  Const iSpurLinks As Inp = Inp.I4

  Const dUltra As Inp = Inp.D1

  Const aLicht As Inp = Inp.AX
  Const aTemp As Inp = Inp.AY

  Const aFarbe As Inp = Inp.A1

  ' --- Farbkonstanten -------

  Const blauMin As Integer = 240
  Const blauMax As Integer = 260
  Const grunMin As Integer = 200
  Const grunMax As Integer = 220
  Const rotMin As Integer = 170
  Const rotMax As Integer = 190

  Const heissMax = 450
  Const lichtMin = 500

  ' --- Unterprogramme Fahren ---------------------

  Private Sub Stopp()
    ft.SetMotor(mLinks, cOff)
    ft.SetMotor(mRechts, cOff)
  End Sub
  Private Sub StoppR()
    ft.SetMotor(mLinks, cVor)
    ft.SetMotor(mRechts, cOff)
  End Sub
  Private Sub StoppL()
    ft.SetMotor(mLinks, cOff)
    ft.SetMotor(mRechts, cVor)
  End Sub
  Private Sub Vorwarts()
    ft.SetMotor(mLinks, cVor, Speed.Half)
    ft.SetMotor(mRechts, cVor, Speed.Half)
  End Sub
  Private Sub Ruckwarts()
    ft.SetMotor(mLinks, cRuck, Speed.Half)
    ft.SetMotor(mRechts, cRuck, Speed.Half)
  End Sub
  Private Sub Links()
    ft.SetMotor(mLinks, cRuck, Speed.Half)
    ft.SetMotor(mRechts, cVor, Speed.Half)
  End Sub

  ' --- Hindernis : Wenn Entfernung zu einem Hindernis größer 12 -> Return
  '     sonst rückwärts bis Entfernung > 12 cm, dann 500 LinksDrehen
  Private Function Hindernis() As Boolean
    If ft.GetDistance(dUltra) > 12 Then Return False
    Stopp()
    ft.Pause(1000)
    Ruckwarts()
    Do Until ft.Finish() Or ft.GetDistance(dUltra) > 12
      ft.Pause(123)
    Loop
    Stopp()
    Links()
    ft.Pause(500)
    Return True
  End Function

  ' --- Spursuche : Wenn ein SpurSensor auf Spur : Return
  '     sonst EndlosSuche 52x High Links und 10x High Vorwärts
  Private Sub Spursuche()
    Dim Zahler As Integer
    AufSpur = False
    Do Until ft.Finish()
      Zahler = 52
      Links()
      Do
        If Not ft.GetInput(iSpurLinks) Or Not ft.GetInput(iSpurRechts) Then
          Stopp()
          AufSpur = True
          Return
        End If
        ft.WaitForHigh(iImpulsLinks)
        If Hindernis() Then
          Zahler = 52
          Links()
        Else
          Zahler -= 1
        End If
      Loop Until Zahler <= 0 Or ft.Finish()
      Zahler = 10
      Vorwarts()
      Do
        If Not ft.GetInput(iSpurLinks) Or Not ft.GetInput(iSpurRechts) Then
          Stopp()
          AufSpur = True
          Return
        End If
        ft.WaitForHigh(iImpulsLinks)
        If Hindernis() Then Zahler = 0 Else Zahler -= 1
      Loop Until Zahler <= 0 Or ft.Finish()
    Loop
  End Sub

  ' --- Thread Main-Routinen -----------------------------------------

  ' --- AutoLoop : Fahren unter Auswertung der Spursensorwerte ----------------

  Private Sub AutoLoop()
    Do Until ft.Finish()
      Select Case ft.GetInputs() And &HC
        Case &HC
          Stopp()
          Spursuche()
        Case &H8
          StoppL()
          Hindernis()
        Case &H4
          StoppR()
          Hindernis()
        Case Else
          Vorwarts()
          Hindernis()
      End Select
    Loop
  End Sub

  ' --- LichtLoop : Schalten des Fahrlichtes - ein bei Helligkeitswerten über lichtMin ---

  Private Sub LichtLoop()
    Do Until ft.Finish()
      If ft.GetAnalog(aLicht) > lichtMin Then
        ft.SetLamp(oLampeL, Dir.On)
        ft.SetLamp(oLampeR, Dir.On)
      Else
        ft.SetLamp(oLampeL, Dir.Off)
        ft.SetLamp(oLampeR, cOff)
      End If
    Loop
  End Sub

  ' --- TempLoop : Temperaturkontrolle - RoteLampe, wenn Wert < heissMax ----------

  Private Sub TempLoop()
    Do Until ft.Finish()
      If ft.GetAnalog(aTemp) < heissMax Then
        ft.SetLamp(oLampeT, cOn)
        ft.Pause(100)
        ft.SetLamp(oLampeT, cOff)
        ft.Pause(600)
      End If
    Loop
  End Sub

  ' --- FarbLoop : Farberkennung, nur wenn Explorer auf Spur ----

  Private Sub Hupen(ByVal nMal As Integer, ByVal Dauer As Integer)
    For i As Integer = 1 To nMal
      ft.SetLamp(oHupe, cOn)
      ft.Pause(Dauer)
      ft.SetLamp(oHupe, cOff)
      ft.Pause(Dauer)
    Next
  End Sub

  Private Sub FarbLoop()
    Do Until ft.Finish()
      If AufSpur Then
        Select Case ft.GetVoltage(aFarbe)
          Case blauMin To blauMax
            Hupen(4, 100)
          Case grunMin To grunMax
            Hupen(3, 300)
          Case rotMin To rotMax
            Hupen(2, 600)
          Case Else
        End Select
      End If
    Loop
  End Sub
  Private Sub Action()
    Do
      lblAbstand.Text = ft.GetDistance(dUltra).ToString()
      lblFarbwert.Text = ft.GetVoltage(aFarbe).ToString()
      lblHelligkeit.Text = ft.GetAnalog(aLicht).ToString()
      lblTemperatur.Text = ft.GetAnalog(aTemp).ToString()
      lblSpurL.Text = IIf(ft.GetInput(iSpurRechts), "F", "T")
      lblSpurR.Text = IIf(ft.GetInput(iSpurLinks), "F", "T")
      ft.Pause(77)
    Loop Until ft.Finish()
  End Sub

#Region "--- ProgramControl ---"
  Private Sub cmdAction_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles cmdAction.Click
    Try
      ft.OpenInterface(IFTypen.ftROBO_first_USB, 0, Distance.UltraSonic, True)
      cmdAction.Enabled = False
      chkAutoPilot.Enabled = True
      pnlDrive.Enabled = True
      cmdEnde.Text = "&HALT"
      lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---"

      Action()

      ft.ClearMotors()
    Catch ex As FishFaceException
      MsgBox(ex.Message)
    Finally
      Thread.Sleep(555)
      ft.CloseInterface()
      cmdAction.Enabled = True
      chkAutoPilot.Enabled = False
      pnlDrive.Enabled = False
      cmdEnde.Text = "&ENDE"
      lblHinweis.Text = "--- Nochmal : ACTION, sonst ENDE oder ESC-Taste ---"
      cmdEnde.Focus()
    End Try

  End Sub

  Private Sub cmdEnde_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdEnde.Click
    If cmdEnde.Text = "&HALT" Then
      ft.NotHalt = True
      chkAutoPilot.Checked = False
    Else
      Me.Close()
    End If
  End Sub

  Private Sub Form_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
    If cmdEnde.Text = "&HALT" Then e.Cancel = True
  End Sub

#End Region

  ' --- Routinen für FahrButtons --------------------------------------------------------

  Private Sub cmdVor_MouseDown(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdVor.MouseDown
    ft.SetMotor(mLinks, cVor, CType(numGeschwindigkeit.Value, Speed))
    ft.SetMotor(mRechts, cVor, CType(numGeschwindigkeit.Value, Speed))
  End Sub

  Private Sub cmdVor_MouseUp(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdVor.MouseUp
    Stopp()
  End Sub

  Private Sub cmdZuruck_MouseDown(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdZuruck.MouseDown
    ft.SetMotor(mLinks, cRuck, CType(numGeschwindigkeit.Value, Speed))
    ft.SetMotor(mRechts, cRuck, CType(numGeschwindigkeit.Value, Speed))
  End Sub

  Private Sub cmdZuruck_MouseUp(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdZuruck.MouseUp
    Stopp()
  End Sub

  Private Sub cmdLinks_MouseDown(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdLinks.MouseDown
    ft.SetMotor(mLinks, cRuck, CType(numGeschwindigkeit.Value, Speed))
    ft.SetMotor(mRechts, cVor, CType(numGeschwindigkeit.Value, Speed))
  End Sub

  Private Sub cmdLinks_MouseUp(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdLinks.MouseUp
    Stopp()
  End Sub

  Private Sub cmdRechts_MouseDown(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdRechts.MouseDown
    ft.SetMotor(mLinks, cVor, CType(numGeschwindigkeit.Value, Speed))
    ft.SetMotor(mRechts, cRuck, CType(numGeschwindigkeit.Value, Speed))

  End Sub

  Private Sub cmdRechts_MouseUp(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdRechts.MouseUp
    Stopp()
  End Sub

  ' --- Routinen Licht und Hupe Buttons ---------------------------------------------

  Private Sub cmdLicht_Click(ByVal sender As System.Object, _
    ByVal e As System.EventArgs) Handles cmdLicht.Click
    If LichtAn Then
      ft.SetLamp(oLampeR, cOff)
      ft.SetLamp(oLampeL, cOff)
      cmdLicht.BackColor = Color.LightBlue
      LichtAn = False
    Else
      ft.SetLamp(oLampeR, cOn)
      ft.SetLamp(oLampeL, cOn)
      cmdLicht.BackColor = Color.LightPink
      LichtAn = True
    End If
  End Sub

  Private Sub cmdHupe_MouseDown(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdHupe.MouseDown
    ft.SetLamp(oHupe, cOn)
  End Sub

  Private Sub cmdHupe_MouseUp(ByVal sender As System.Object, _
    ByVal e As System.Windows.Forms.MouseEventArgs) Handles cmdHupe.MouseUp
    ft.SetLamp(oHupe, cOff)
  End Sub

  Private Sub chkAutoPilot_CheckedChanged(ByVal sender As System.Object, _
    ByVal e As System.EventArgs) Handles chkAutoPilot.CheckedChanged
    If chkAutoPilot.Checked Then
      pnlDrive.Enabled = False
      ft.NotHalt = False
      chkAutoPilot.BackColor = Color.Red
      AutoPilot = New Thread(AddressOf AutoLoop)
      AutoPilot.IsBackground = True
      Licht = New Thread(AddressOf LichtLoop)
      Licht.IsBackground = True
      Temperatur = New Thread(AddressOf TempLoop)
      Temperatur.IsBackground = True
      Farberkennung = New Thread(AddressOf FarbLoop)
      Farberkennung.IsBackground = True
      AutoPilot.Start()
      Licht.Start()
      Temperatur.Start()
      Farberkennung.Start()
    Else
      ft.NotHalt = True
      pnlDrive.Enabled = True
      chkAutoPilot.BackColor = Color.Gray
      Thread.Sleep(123)
      ft.ClearMotors()
    End If
  End Sub
End Class
